; compile with MPLAB X IDE mpasm or with gpasm assembler.
; gpasm -a inhx32 -p p18f47k40 PIC18F47K40_64MHz_boot.asm
;*******************************************************************************
; This bootloader is used used with the Linux terminal program sendhex.
; To use the bootloader, program Files need to be compiled with a code offset 
; of 0x0004 or ORG 0x0004. The goto bootloader instruction is at 00000-00003.
; At exit, the booloader goes to 0x00004 to find user program	
; The bootloader occupies the upper 256 bytes of flash (two 128 byte blocks).
;
; The boot program was writen for PIC18F47K40 attributes:
; 1. Using the register names for 47K40	
; 2. The 47K40 serial port pins configured as digital
; 3. The EUSART peripherals assigned to specific port pins
; 4. It writes Flash Program Memory in 128 byte blocks
; 5. EEprom and Config addressing changed for 47K40 memory map
; 
;    Use ./sendhex to upload .hex files
;  
; Debugging:
;   To aid debugging, the bootloader can be compiled with "#define feedback"  
;   when this is defined, data received by the bootloader will be sent out
;   by UART2 and can be monitored on a terminal program such as "minicom"
;   connected through a TTL to serial or TTL to USB adapter.	
;*******************************************************************************
; Credit is given to: claudiu.chiculita@ugal.ro
;   http://www.etc.ugal.ro/cchiculita/software/picbootloader.htm
; The bootloader is based on the general outline of a tinybldln bootloader	
;*******************************************************************************	
		
	radix DEC
	LIST  P=18F47K40  ; change also: Configure->SelectDevice from Mplab
       	        
	; #define feedback           ; "feedback" echos received data to USART2
	#define flash_size  0x20000  ; 131072 bytes of program flash memory
        #include <p18f47k40.inc>     ; register definitions
	ifdef feedback
	#define first_address flash_size-258 ;256 bytes = size of bootloader
        else
	#define first_address flash_size-222 ;220 bytes = size of bootloader
	endif
		
	CONFIG FEXTOSC = OFF, RSTOSC = HFINTOSC_64MHZ, WDTE = OFF   
	;---------------list of 47K40 config settings---------------
	
	;CONFIG FEXTOSC = OFF, RSTOSC = HFINTOSC_64MHZ  
	;CONFIG CLKOUTEN = OFF, CSWEN = ON, FCMEN = ON  
        
	;CONFIG MCLRE = EXTMCLR, PWRTE = OFF, LPBOREN = OFF, BOREN = SBORDIS  L
	;CONFIG BORV = VBOR_2P45, ZCD = OFF, PPS1WAY = ON, STVREN = ON, DEBUG = OFF, XINST = OFF 
	
	;CONFIG WDTCPS = WDTCPS_31, WDTE = OFF 
	;CONFIG WDTCWS = WDTCWS_7, WDTCCS = SC 
	
	;CONFIG WRT0 = OFF, WRT1 = OFF, WRT2 = OFF, WRT3 = OFF, WRT4 = OFF, WRT5 = OFF, WRT6 = OFF, WRT7 = OFF 
	;CONFIG WRTC = OFF, WRTB = OFF, WRTD = OFF, SCANE = ON, LVP = ON 
        
	;CONFIG CP = OFF, CPD = OFF 
	
	;CONFIG EBTR0 = OFF, EBTR1 = OFF, EBTR2 = OFF, EBTR3 = OFF, EBTR4 = OFF, EBTR5 = OFF, EBTR6 = OFF, EBTR7 = OFF  	
	;CONFIG EBTRB = OFF 
	
;----------------------------- PROGRAM ---------------------------------
;---------------------RAM variables declared----------------------------
	cblock 0
	crc         ; all received chars added to crc when last char added = 0
	blkcount    ; count chars received
	cnt1        ; delay counter for receive timeout
	cnt2	    ; delay counter for receive timeout
	cnt3	    ; delay counter for receive timeout
	tblcount    ; size of table block to be programmed
	flag        ; Holds Upper address to flag EEPROM or CONFIG
	endc 
	cblock 10
	buffer:128  ; buffer for received bytes
	endc
	
;---------------------MACRO to send data to sendhex uploader------------	
SendByte macro car
	movlw car
	movwf TX1REG   ; transmit byte to host- sendasm
	endm
	
;0000000000000000000000000 RESET 00000000000000000000000000

	ORG  0x0000    ; goto bootloader instruction put here
	GOTO bootloader

;view with TabSize=4
;&&&&&&&&&&&&&&&&&&&&&&&   START     &&&&&&&&&&&&&&&&&&&&&&
;----------------------  Bootloader  ----------------------
	
	ORG first_address 	
	goto 0x00004            ;goto start of user program  
	
bootloader                

	;--------added for 18f47k40
	;------set port registers to digital------------------------- 				
       	
	movlb 15
	clrf ANSELC		;15B) ANSELC = 0  set PORTC to digital (rx and tx)	
	clrf ANSELD		;     ANSELD = 0  set PORTD to digital (rx and tx)
	
	;------SETUP EUSART1 --------------------------------------
	
	movlb 14
	movlw 0x17		;  RX1PPS = 0x17   RECEIVE 1 SET TO RC7       
	movwf RX1PPS		;14B)
	movlw 0x09		;  RC6PPS = 0x09   RC6 set to Transmit 1 output
        movwf RC6PPS		;14B)
	movlw 0x08		;  BAUD1CON = 0X08 ABDOVF RCIDL - SCKP BRG16 - WUE ABDEN
        movwf BAUD1CON          ;ACC)                  0      0   0   0    1   0  0    0

	clrf SP1BRGH		;ACC) SP1BRGH = 0X00  set baud rate generator to 115200 baud
	movlw 0x22		;   SP1BRGL = 34    ((64000000 / 115200) / 16  ) - 1 = 33.722
	movwf SP1BRGL		;ACC)
	movlw 0x20		;  TX1STA = 0X20   CSRC  TX9 TXEN  SYNC SEND BRGH TRMT TX9D
	movwf TX1STA		;ACC)                0    0    1     0    0    0   = transmit enable
  
	movlw 0x90		;  RC1STA = 0X90   SPEN RX9 SREN CREN ADDEN FERR OERR RX9D
	movwf RC1STA		;ACC)                1   0    0    1  = serial port & continuous receive 
	
        ;------SETUP EUSART2 - (optional- only if "feedback" defined) ------------------

	
        ifdef feedback          ; comment out "#define feedback" if echo not wanted
        movlb 14
	movlw 0x1F		;  RX2PPS = 0x1F  Receive 2 SET TO RD7       
	movwf RX2PPS		;14B)
	movlb 15
	movlw 0x0B		;  Rd6PPS = 0x0B   RD6 set to Transmit 2 output
        movwf RD6PPS		;15B)
	movlb 14
	movlw 0x08		;  BAUD2CON = 0X08 ABDOVF RCIDL - SCKP BRG16 - WUE ABDEN
        movwf BAUD2CON          ;14)                 0      0   0   0    1   0  0    0

	clrf SP2BRGH		;  SP2BRGH = 0X00  set baud rate generator to 115200 baud
	movlw 0x22		;  SP2BRGL = 34    ((64000000 / 115200) / 16  ) - 1 = 33.722
	movwf SP2BRGL		;14)
	movlw 0x20		;  TX2STA = 0X20   CSRC  TX9 TXEN  SYNC SEND BRGH TRMT TX9D
	movwf TX2STA		;14)                0    0    1     0    0    0   = transmit enable
  
	movlw 0x90		;  RC1STA = 0X90   SPEN RX9 SREN CREN ADDEN FERR OERR RX9D
	movwf RC2STA		;14)                  1   0    0    1  = serial port & continuous receive
	endif

	;-------------------comm ports configured ------------------------- 				
        ;-----sendhex waits for this 'R' to be sent by reseting board -----
        SendByte 'R'            ; send an R for reset to sendhex uploader
	
	;-----wait 1 sec. for sendhex to send C1h 
	rcall Receive			
	sublw 0xC1		; Expect C1h
	bnz   exit_bootloader   ; if received data not C1h then exit
	
        ;------------------reset sent--------------------------
	;------receive .hex file code address data----------------
mainloop
	SendByte 'K'		; "-Everything OK, ready to receive data"
main	
	clrf  crc		; clear crc to 0

	rcall Receive		; Upper address (00 0000-00 0001 = pgm)
	movwf TBLPTRU		; put in table pointer upper (bits 0-5)
	movwf flag		; for CFG and EEPROM cases (11 0000-11 0001)

	rcall Receive		; High 8 bits of address
	movwf TBLPTRH		; put in table pointer high
	movwf NVMADRH		; NVMADRH 8 high address bits
	
	rcall Receive		; Low 8 bits of address
	movwf TBLPTRL		; put in table pointer low
	movwf NVMADRL           ; ****
	rcall Receive		; receive block byte count (128 bytes,80h)
	movwf blkcount          ; move to blkcount
	incf blkcount           ; increment counter by 1 to include crc (= 65)

      	;----- address and block size received ---------------------
	;----- receive a block of .hex file program code -----------
	
	lfsr FSR0, buffer-1     ; move address of (buffer-1) to FSR0
	
recblk	; begin reading blkcount #of bytes sent 
	
	movwf NVMDAT            ; EE data from Receive below on loop to recblk
	movwf TABLAT		;15B store byte
	rcall Receive           ; start getting program bytes - in w reg
	movwf PREINC0		;14B store received data 
	decfsz blkcount         ; count down bytes recieved (from 65) skip if 0
	bra recblk              ; loop to rcoct until up to 129 bytes received
	
	;---------- 1 block of blkcount # of bytes received & loaded ----------
	;-----check if crc is valid (last byte added should zero crc)----------  
	
	tstfsz crc		; check crc skip if zero
	bra crcerror            ; branch to crcerror if not zero 
	;----------crc of 1st block OK----------------

	SendByte 'K'		; acknowledge 1st block received OK

	;-----check if block is EEPROM, Config, or UserID data --------
 	btfss flag,5             ; *is flag eeprom config or user ID? 001x 0000
        bra isprogram            ; *if bit 5 = 0 non of non above 
	
	;---if EEPROM, Config or UserID - test if it is EEPROM-------  
	btfss flag,0             ; *test if bit 0 is a 1 - 001x 0001
	bra isCFGorID            ; *if bit 0 = 0 then it is config of UserID
	
	;----------if EEPROM data - write eeprom -----------
	movlw b'00000100'	; Setup eeprom write
	rcall Write             ; cal write  to write buffer to eeprom
	bra waitwrite           ; branch to wait for write operation to finish
	;(then go on to mainloop from waitwrite)
	;------------eeprom written-------------------------
	
isCFGorID 			 
	;-----write config or UserID -----------------------
	tblwt*			; write TABLAT(byte before crc) to TBLPTR*
	movlw b'11000100'	; Setup write to configuration bits
	rcall Write             ; write table to config addresses
	bra waitwrite             ; branch to wait for write to finish
	;(then go on to mainloop from waitwrite)
        ;----------- config or UserID written--------------------
	
	;----- erase 128 byte block of Program Flash Memory -----	
isprogram                       ; was noconfig
	movlw b'10010100'	; Setup erase flash
	rcall Write             ; erase 64 bytes of flash 
waiterase	
	btfsc NVMCON1,WR	; test w bit in nvmcon1 skip if 0
	bra waiterase		; wait to finish erase
	TBLRD*-			; dummy read decrement
        ;------------flash erased--------------------------------
	
	;----- write 128 byte block of Program Flash Memory------		
	lfsr FSR0, buffer	; put memory address of buffer in fsr0
writeloop
	movlw 128		; 128 bytes to transfer from buffer table latch
	movwf tblcount          ; put 128 count in tblcount
writebyte
	movf POSTINC0,w		; put 1 byte from buffer (FSR0) into table
	movwf TABLAT
	tblwt+*                 ; table write with post increment
	decfsz tblcount         ; continue for 64 writes to table
	bra writebyte           ; goto write table to flash
	
	;------------data transfered from buffer to table-------
	;-----write table to flash------------------------------
	
	movlw b'10000100'	; Setup writes to flash
	rcall Write             ; got write table to flash
	
	;------wait for write sequence to end------------------	
waitwrite				; NVMREG<1:0> - FREE WRERR WREN WR RD
	btfsc NVMCON1,WR	; wait for write to finish write)
	bra waitwrite		; branch until write complete
	
	;------------table written to flash---------------------	
	bcf NVMCON1,WREN	; disable writes
	bra mainloop            ; branch to get next block of .hex file data
	
	;--------------end of mainloop---------------------------
	;-----crc failed - send an 'N' - Not OK acknowledge------	
crcerror			; CRC failed
	SendByte 'N' 
	bra main		; branch to main skips the OK acknowledge
	  
	;---------------subroutines --------------------
	;-----write non volatile memory subroutine------
Write                           ; write data in 128 byte table to flash
	movwf NVMCON1           ; put bxx0x0100 in nvcon1
	movlw 0x55              ; run nvm unlock sequence
	movwf NVMCON2            
	movlw 0xAA
	movwf NVMCON2		 
	bsf NVMCON1,WR		; initiate WRITE (2ms)
	nop			; nop					
	return

        ;------------write nvm initiated ------------------
       	;------ receive serial data subroutine ------------
Receive
	movlw 32	        ; @ 1 sec delay - wait time for received data 
	movwf cnt1		
rpt2						
	clrf cnt2
rpt3
	clrf cnt3
rptc
	movlb 14		; switch to bank 14 for PIR3 register
	btfss PIR3, RC1IF	;14B test RX receieved
	bra noreceive		; goto delay - noreceive
	movf RC1REG,w		; move received byte to W register
        ifdef feedback
	movwf TX2REG            ;14 echo received character to TX2REG
	endif
	addwf crc,f		; add received byte to crc 
	return			; return with received byte in w register

        ;--------------data recieved - return----------------------
        ;----- receive wait delay---------------------------
noreceive
	decfsz cnt3		; wait for receive or timeout sequence
	bra rptc		; branch to check receive flag
	decfsz cnt2
	bra rpt3		; branch to clr cnt2,cnt3 then check receive flag
	decfsz cnt1             ; decrement skip if 0 = timed out so exit
	bra rpt2                ; branch to clr cnt3 then check receive flag
	
	;----- Receive timed out - no more data - exit--------
	
exit_bootloader
	bcf	RC1STA,	SPEN	; deactivate UART 1
	ifdef feedback
	bcf     RC2STA, SPEN    ; deactivate UART 2
	endif
	bra first_address       ;goto start of user program           
	
	END
	
        
