// EUSART1 interrupt tested OK
// TIMER0 toggle tested OK
// compile with MPLABX code offset of 0x0004 

#include <xc.h>

#define _XTAL_FREQ 64000000  // needed for __delay_ms())

// eeprom data example: each __eeprom_data(); block has to be 8 bytes long
__EEPROM_DATA(0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07); // address 31 0000 - 31 0007

        
// set Config 1L to internal 64 MHz clock, 3L to watch dog timer enable off
#pragma	config CONFIG1L FEXTOSC = OFF, RSTOSC = HFINTOSC_64MHZ  // 18f47k40
#pragma	config CONFIG3L WDTE = OFF

volatile char ch = 0;


void interrupt ISR() //__at( 0x0008 )// Main Interrupt Service Routine (ISR)
{        
  // if( RC1IF )      // (PIE3) RC2IF TX2IF RC1IF TX1IF BCL2IF SSP2IF BCL1IF SSP1IF
     ch = RC1REG;     // ECHO                (1)
     TX1REG = ch;
     TX2REG = ch;                             
    // if( TMR0IF )   // PIR0 TEST BIT 5 == 1 = TMR0iF    
    //  PORTBbits.RB5 = ~PORTBbits.RB5;
       
   GIE = 1;           // Global Interrupt Enable
    //TMR0IF = 0;     // TMR0IF = CLEAR INTERRUPT FLAG
}


void main()  //__at( 0x0004) // locates main at address 0x0000
{ 
// SETUP PORTS B, C and D 
  ANSELB = 0x00;    //  set PORTB to digital
  ANSELC = 0x00;    //  set PORTC to digital
  ANSELD = 0x00;    //  set PORTD to digital
  TRISB5 = 0;       //  set the B register bit 5 to output
  TRISC7 = 1;       //  set receive C7 to input  
  TRISC6 = 0;       //  set transmit C6 to output
  TRISD7 = 1;       //  set receive D7 to input  
  TRISD6 = 0;       //  set transmit D6 to output  

// SETUP EUSART1-----------------------------------------------------------
  
  RX1PPS = 0x17;    // RECEIVE 1 SET TO RC7
  RC6PPS = 0x09;    // RC6 set to Transmit 1 output
 
  BAUD1CON = 0X08;  // ABDOVF RCIDL - SCKP BRG16 - WUE ABDEN
                    //   0      0   0   0    1   0  0    0

  SP1BRGH = 0X00;   // set baud rate generator to 115200 baud
  SP1BRGL = 34;     // ((64000000 / 115200) / 16  ) - 1 = 33.722

  TX1STA = 0X20;    // CSRC  TX9 TXEN  SYNC SEND BRGH TRMT TX9D
                    //   0    0    1     0    0    0   = transmit enable
  
  RC1STA = 0X90;    // SPEN RX9 SREN CREN ADDEN FERR OERR RX9D
                    //   1   0    0    1  = serial port & continuous receive enable 
  RC1IE = 1;        // enable receive interrupt in PIE3 register
//-------------------------------------------------------------------------

// SETUP EUSART2-----------------------------------------------------------
  
  RX2PPS = 0x1F;    // RECEIVE 2 SET TO RD7  
  RD6PPS = 0x0B;    // RD6 set to Transmit 2 output
 
  BAUD2CON = 0X08;  // ABDOVF RCIDL - SCKP BRG16 - WUE ABDEN
                    //   0      0   0   0    1   0  0    0

  SP2BRGH = 0X00;   // set baud rate generator to 115200 baud
  SP2BRGL = 34;     // ((64000000 / 115200) / 16  ) - 1 = 33.722

  TX2STA = 0X20;    // CSRC  TX9 TXEN  SYNC SEND BRGH TRMT TX9D
                    //   0    0    1     0    0    0   = transmit enable
  
  RC2STA = 0X90;    // SPEN RX9 SREN CREN ADDEN FERR OERR RX9D
                    //   1   0    0    1  = serial port & continuous receive enable 
  // RC2IE = 1;     // enable receive interrupt in PIE3 register  
// ------------------------------------------------------------------------

// SETUP TIMER 0 - 8 BIT MODE - toggle RB5 on overflow --------------------
  
  RB5PPS = 0x13;    // B5 set to TIMER0 output - toggles on TIMER0 rollover
  
  T0CON0 = 0x8F;    // T0EN  -  T0OUT  T016BIT   T0OUTPS<3:0>
                    //   1   0   (R)      0       1 1 1 1 = 1/16 POSTSCALER
  T0CON1 = 0x7F;    //  T0CS<2:0>  T0ASNC   T0CKPS<3:0>                  
         	    //   0  1  1     1      1  1  1  1 = 1/32768 PRESCALLER (122) 
  TMR0H = 61;       // set time to 1 sec. blink
                 
  TMR0MD = 0;       // PMD CONTROL REGISTER BIT 0 TMR0MD = 0 = TIMER 0 MODULE ENABLED
  
  // TMR0IP = 0;    // INTERRUPT PROIRITY OF TIMER 0, TMR0IP 0 = LOW 1 = HIGH
  // TMR0IE = 1;    // SET BIT 5 TO ENABLE TIMER0 INTERRUPT (in PIE0 register)
//-------------------------------------------------------------------------

// enable peripheral and global interrupts --------------------------------
  
  PEIE = 1;         // Enable Peripheral Interrupt (in INTCON register)
  GIE = 1;          // Enable INTs globally (in INTCON register)
//-------------------------------------------------------------------------

  while(1)
    {}              // do nothing - interrupts handles EUSART1     
  return;
}
